import mpl_toolkits
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import astropy.units as u


def plot_pointings(pointings, ax=None, projection='polar', add_grid3d=False, **kwargs):
    """
    Produce a scatter plot of the pointings

    Parameters
    ----------
    pointings: 2D array of `astropy.quantities` or numpy array in rad
    ax : `matplotlib.pyplot.Axis`
    projection: str or None
        '3d' | 'aitoff' | 'hammer' | 'lambert' | 'mollweide' | 'polar' | 'rectilinear'
    add_grid3d: bool
        add a 3D grid in case of projection='3d'
    kwargs: dict
        kwargs for `matplotlib.pyplot.scatter`

    Returns
    -------
    ax: `matplotlib.pyplot.axis`
    """

    if ax and projection and not isinstance(ax, matplotlib.projections.get_projection_class(projection)):
        raise ValueError(f"ax of type {type(ax)} and projection {projection} are exclusive")

    if ax is None:
        fig = plt.gcf()
        ax = fig.add_subplot(111, projection=projection)

    elif isinstance(ax, mpl_toolkits.mplot3d.axes3d.Axes3D):
        projection = '3d'
    elif isinstance(ax, matplotlib.projections.polar.PolarAxes):
        projection = 'polar'

    if projection == '3d':
        r = 1.0
        if add_grid3d:
            az, alt = np.mgrid[0 : 2.01 * np.pi : 0.1 * np.pi / 2.0, 0 : np.pi / 2.0 : 0.1 * np.pi / 4.0]  # noqa
            x = r * np.cos(az) * np.cos(alt)
            y = r * np.sin(az) * np.cos(alt)
            z = r * np.sin(alt)
            ax.plot_surface(x, y, z, cmap=plt.cm.YlGnBu_r, alpha=0.1)
        r *= 1.01
        az = pointings[:, 0]
        alt = pointings[:, 1]
        x = r * np.cos(az) * np.cos(alt)
        y = r * np.sin(az) * np.cos(alt)
        z = r * np.sin(alt)

        ax.scatter(x, y, z, **kwargs)

        box_ratio = 1.03
        ax.set_xlim3d([-r * box_ratio, r * box_ratio])
        ax.set_ylim3d([-r * box_ratio, r * box_ratio])
        ax.set_zlim3d([-r * box_ratio, r * box_ratio])

    elif projection == 'polar':
        ax.scatter(pointings[:, 0], np.pi / 2.0 * u.rad - pointings[:, 1], **kwargs)
        ax.set_xlabel('Azimuth')
        rticks_deg = [10, 30, 50, 70, 90]
        ax.set_rticks(np.deg2rad(rticks_deg), labels=[f'{r}°' for r in rticks_deg])
        ax.set_rmax(np.pi / 2.0)
        ax.set_rlabel_position(20)

    else:
        ax.scatter(pointings[:, 0], pointings[:, 1], **kwargs)
        ax.set_xlabel('Azimuth')
        ax.set_ylabel('Altitude')

    ax.legend()
    ax.grid(True)
    ax.set_axisbelow(True)

    return ax



#!/usr//bin/env python3

import logging
from pathlib import Path
from ..utils import save_log_to_file, SbatchLstMCStage

log = logging.getLogger(__name__)


def batch_merge_dl1(dict_paths, batch_config, logs, jobid_from_splitting, workflow_kind="lstchain"):
    """
    Function to batch the onsite_mc_merge_and_copy function once the all the r0_to_dl1 jobs (batched by particle type)
    have finished.

    Batch 8 merge_and_copy_dl1 jobs ([train, test] x particle) + the move_dl1 and move_dir jobs (2 per particle).

    Parameters
    ----------
    dict_paths : dict
        Core dictionary with {stage: PATHS} information
    batch_config : dict
        Dictionary containing the (full) source_environment and the slurm_account strings to be passed
        to `merge_dl1` and `compose_batch_command_of_script` functions.
    workflow_kind : str
        Defines workflow kind (lstchain, ctapipe, hiperta)
    logs: dict
        Dictionary with logs files
    jobid_from_splitting: str

    Returns
    -------
    jobids_for_train : str
         Comma-separated str with all the job-ids to be passed to the next
         stage of the workflow (as a slurm dependency)

    """
    log_merge = {}
    all_jobs_merge_stage = []
    debug_log = {}
    log.info('==== START batch merge_and_copy_dl1_workflow ====')
    for paths in dict_paths:
        job_logs, jobid_debug = merge_dl1(
            paths["input"],
            paths["output"],
            merging_options=paths.get('options', None),
            batch_configuration=batch_config,
            wait_jobs_split=jobid_from_splitting,
            workflow_kind=workflow_kind,
            extra_slurm_options=paths.get("extra_slurm_options", None),
        )

        log_merge.update(job_logs)
        all_jobs_merge_stage.append(jobid_debug)
    save_log_to_file(log_merge, logs["log_file"], "merge_dl1")
    save_log_to_file(debug_log, logs["debug_file"], workflow_step="merge_dl1")
    log.info('==== END batch merge_and_copy_dl1_workflow ====')
    return ','.join(all_jobs_merge_stage)


def merge_dl1(
    input_dir,
    output_file,
    batch_configuration,
    wait_jobs_split="",
    merging_options=None,
    workflow_kind="lstchain",
    extra_slurm_options=None,
):
    """

    Parameters
    ----------
    input_dir: str
    output_file: str
    batch_configuration: dict
    wait_jobs_split: str
    merging_options: dict
    workflow_kind: str
    extra_slurm_options: dict
        Extra slurm options to be passed to the sbatch command

    Returns
    -------
    log_merge: dict
    jobid_merge: str

    """
    merging_options = "" if merging_options is None else merging_options
    if workflow_kind in ["lstchain", "hiperta"]:
        cmd = f'lstchain_merge_hdf5_files -d {input_dir} -o {output_file} {merging_options}'

    else:
        cmd = f'ctapipe-merge --input-dir {input_dir} --output {output_file} {merging_options}'

    sbatch_merge_dl1 = SbatchLstMCStage(
        "merge_dl1",
        wrap_command=cmd,
        slurm_error=Path(output_file).parent.joinpath("merging-output.e"),
        slurm_output=Path(output_file).parent.joinpath("merging-output.o"),
        slurm_dependencies=wait_jobs_split,
        extra_slurm_options=extra_slurm_options,
        slurm_account=batch_configuration["slurm_account"],
        source_environment=batch_configuration["source_environment"],
    )

    jobid_merge = sbatch_merge_dl1.submit()
    log_merge = {jobid_merge: sbatch_merge_dl1.slurm_command}
    log.info(f"\nMerging DL1 file from {input_dir} dir into {output_file} file.")
    log.info(f"Submitted batch job {jobid_merge}")
    return log_merge, jobid_merge
